#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <inttypes.h>

#define F_CPU 3686400L  // crystal

#define TX_BUFFER_SIZE 32

#define UART_BAUD_RATE 19200L //9600
#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)


// Globale Variablen
// UART
static char TxBuff[TX_BUFFER_SIZE];
static volatile uint8_t TxRead;		// Zeigt auf Zeichen, das gesendet werden soll
static volatile uint8_t TxWrite;		// Zeigt auf nchste Schreibposition im Puffer

static int output = 0;

#define MAX_V 1023
#define MIN_V 380


#define MAX(x,y) ((x) >= (y) ? (x) : (y))

#define MIN(x,y) ((x) <= (y) ? (x) : (y))


void UARTInit()
{ 

  UCSRB = _BV(RXEN) | _BV(TXEN) | _BV(RXCIE); 

  UBRRH = (uint8_t) (UART_UBRR_CALC(UART_BAUD_RATE,F_CPU) >> 8);
  UBRRL = (uint8_t)  UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);
 	
  // Flush Receive-Buffer
  do {
   uint8_t dummy;
   (void) (dummy = UDR);
  }
  while (UCSRA & (1 << RXC));
}



void UartPutC(char data) {
	while(TxWrite != TxRead);								// Warte bis Platz im Puffer
	TxBuff[TxWrite++] = data;
	if(TxWrite >= TX_BUFFER_SIZE) TxWrite = 0;
	UCSRB |= (1<<UDRIE);									// UDRE Interrupt ein
}

void UartPutS(const char *s) {
	while(*s) UartPutC(*s++);
}


// Interrupt Service Routinen
ISR(USART_UDRE_vect) {
	if(TxRead == TxWrite) UCSRB &= ~(1<<UDRIE);			// UDRE Interrupt aus
	else {
		UDR = TxBuff[TxRead++];							// nchtes Zeichen aus Puffer senden
		if(TxRead >= TX_BUFFER_SIZE) TxRead = 0;
	} 							
}


unsigned char UARTGetChar()
{
  	int i=1000;
	while ( (!(UCSRA & (1<<RXC))) && (i > 0) ) i--;
	if (i == 0) return 0;
  	return UDR;                   
}




ISR (USART_RXC_vect)
{
  unsigned char ch;    
  char c2;
  int m1 = OCR1A;
  int m2 = OCR1B;
  uint32_t v;
  int i = 0;
  char s[10];
  
  ch = UARTGetChar();
  //UartPutS("d");
  
  switch (ch) {
    case 'm': 
		// PWM: "m1=1023*"
		ch = UARTGetChar();			
		c2 = UARTGetChar();
		if (c2 != '=') break;
		for (i=0; i < 10; i++){
			s[i] = UARTGetChar();
			if (s[i] == '*') {
				s[i] = '\0';
				break;
			}
		}
		v = strtol(s, NULL, 0);
		if (ch == '1') 
			m1 = (int)v;
		else if (ch == '2')
		    m2 = (int)v;
		output=1;
		break;

	case 'i': 
			output=1;
			break;
	case '1':
			m1 += 10;			
			output=1;
			break;
	case '2':
			m1 -= 10;
			output=1;
			break;		
	case '3':
			m2 += 10;
			output=1;
			break;
	case '4':
			m2 -= 10;
			output=1;
			break;		
    default:             
			break;
  }
  OCR1A = MAX(MIN_V, MIN(MAX_V, m1));
  OCR1B = MAX(MIN_V, MIN(MAX_V, m2));
}



void my_delay(int ms){
	int y=0;
	for(y=0;y<ms;y++){
		_delay_ms(10);
	}
}


main(){
	char s[10];
	int x=0;

	DDRD=0x00;   //D-Pins als Eingnge
	PORTD=0x00;  //PullUps aus
	
    DDRB=0xFF;    //B-Pins als Ausgnge

	OCR1A = 640;  // steuert Motor 1
	OCR1B = 733;  // steuert Motor 2

	ICR1=10000;
	TCCR1A|=(0<<COM1A0)|(1<<COM1A1)|(0<<COM1B0)|
                      (1<<COM1B1)|(0<<FOC1A)|(0<<FOC1B)|
                      (1<<WGM11)|(0<<WGM10);
	TCCR1B|=(0<<ICNC1)|(0<<ICES1)|(1<<WGM13)|(1<<WGM12)|
                      (0<<CS12)|(1<<CS11)|(0<<CS10);

	UARTInit();

	sei();

	my_delay(100);

	UartPutS("start\r\n");


	for(;;){
		// -------------------- X-ACHSE ------------------------
                // Die Leitungen vom Parallel-Port kommen bei
                // PD0 und PD1 rein
                // ----------------------------------------------------------
		/*if(!bit_is_set(PIND,0) && (bit_is_set(PIND,1))){
				OCR1A += 10;
				my_delay(50);
		}

		if(!bit_is_set(PIND,1) && (bit_is_set(PIND,0))){
				OCR1A -= 10;
				my_delay(50);
		}
		// -----------------------------------------------------

		// -------------------- Y-ACHSE ------------------------
                // Leitungen vom ParPort an PD2 und PD3
                // ----------------------------------------------------------
		if(!bit_is_set(PIND,2) && (bit_is_set(PIND,3))){
				OCR1B += 10;
				my_delay(50);
		}

		if(!bit_is_set(PIND,3) && (bit_is_set(PIND,2))){
				OCR1B -= 10;
				my_delay(50);
		}*/
		// -----------------------------------------------------
	  if (output){
	  	output=0;
	  	itoa(OCR1A, s, 10);
  		UartPutS("m1=");
  		UartPutS(s);  
  		itoa(OCR1B, s, 10);
  		UartPutS("&m2=");
  		UartPutS(s);
  		UartPutS("\r\n");
  		//my_delay(500);		
	  }

	}

	return 0;
}

