
// wwwspy Controller
// uC: ATMEGA168  (16 KB Flash, 512 byte EEPROM, 1 KB SRAM)
// EXTERNAL CLOCK: 3.6864 Mhz
//
// IMPORTANT: DO NOT FORGET TO WRITE THE FUSE BIT (via AVR Studio)
//            SO THAT THE EXTERNAL CLOCK IS USED !!


// ----- pins -------------------------
#define PORT_SERVO				PORTB
#define DDR_SERVO				DDRB
#define	P_SERVO1				PB1 
#define P_SERVO2				PB2 
// -----------------------------------


#define F_CPU 3686400L  // crystal

#define TX_BUFFER_SIZE 32


#include <avr/delay.h>
#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <inttypes.h>
// #include <avr/pgmspace.h>

//#include "uart.h"
#include <inttypes.h>
#include <avr/io.h>
//#include <util/delay.h>
//#include <util/delay_basic.h>
//#include "i2cmaster.h"


#define UART_BAUD_RATE 19200L //9600

#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)



// Globale Variablen
// UART
static char TxBuff[TX_BUFFER_SIZE];
static volatile uint8_t TxRead;		// Zeigt auf Zeichen, das gesendet werden soll
static volatile uint8_t TxWrite;		// Zeigt auf nchste Schreibposition im Puffer


static volatile uint32_t timerTicks = 0;




void UARTInit()
{ 

  UCSR0B = _BV(RXEN0) | _BV(TXEN0) | _BV(RXCIE0); 

  UBRR0H = (uint8_t) (UART_UBRR_CALC(UART_BAUD_RATE,F_CPU) >> 8);
  UBRR0L = (uint8_t)  UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);

 	
  // Flush Receive-Buffer
  do {
   uint8_t dummy;
   (void) (dummy = UDR0);
  }
  while (UCSR0A & (1 << RXC0));
}



void UartPutC(char data) {
	while(TxWrite != TxRead);								// Warte bis Platz im Puffer
	TxBuff[TxWrite++] = data;
	if(TxWrite >= TX_BUFFER_SIZE) TxWrite = 0;
	UCSR0B |= (1<<UDRIE0);									// UDRE Interrupt ein
}

void UartPutS(const char *s) {
	while(*s) UartPutC(*s++);
}

/*void UartPutP(const char *progmem_s) {
	register char c;
	while((c = pgm_read_byte(progmem_s++))) UartPutC(c);
}*/

// Interrupt Service Routinen
ISR(USART_UDRE_vect) {
	if(TxRead == TxWrite) UCSR0B &= ~(1<<UDRIE0);			// UDRE Interrupt aus
	else {
		UDR0 = TxBuff[TxRead++];							// nchtes Zeichen aus Puffer senden
		if(TxRead >= TX_BUFFER_SIZE) TxRead = 0;
	} 							
}


/*void UARTPutChar(unsigned char sign)
{
  while (!(UCSRA & (1<<UDRE))); // warten bis Senden moeglich                   
  UDR = sign;                    // schreibt das Zeichen x auf die Schnittstelle
}*/


/*
void UARTWrite(char* msg)
{
  while(*msg != '\0') {
     UARTPutChar (*msg++);
  }
}*/


// Timer1 Overflow interrupt
ISR (TIMER1_OVF_vect)	
{    
	timerTicks++;
}


unsigned char UARTGetChar()
{
  	int i=1000;
	while ( (!(UCSR0A & (1<<RXC0))) && (i > 0) ) i--;
	if (i == 0) return 0;
  	return UDR0;                   
}




ISR (USART_RX_vect) 
//void HandleUART(void)
{
  unsigned char ch;  
  
  ch = UARTGetChar();
  UartPutS("d");
  
  switch (ch) {
    case 'g': 
			UartPutS("h");			
			break;
	case '1':
			if (OCR1A < 1013) OCR1A+=10;
			if (OCR1B < 1013) OCR1B+=10;
			break;
	case '2':
			if (OCR1A > 10) OCR1A-=10;
			if (OCR1B > 10) OCR1B-=10;
			break;		
    default:             
			break;
  }

}


// ----------------------------------------------------------

void initPorts()
{
  // ------------ PWM ----------------------------------------
  // PB1: MotorA OUT2
  // PB2: MotorB OUT2
  TCCR1A = _BV(WGM10) | _BV(WGM11) | _BV(COM1A1) | _BV(COM1B1);
  TCCR1B |= _BV(CS10); // | _BV(ICES1); // 222 Hz PWM
  // Set PWM value 
  OCR1A = 511;  
  OCR1B = 511;
  TCNT1=0;
  // Enable OC1A, OC1B as output. 
  DDR_SERVO |= _BV(P_SERVO1);
  DDR_SERVO |= _BV(P_SERVO2);
  // Enable timer 1 overflow and input capture interrupt. 
  TIMSK1 = _BV (TOIE1); // | _BV(TICIE1);

  // ----------- external interrupt --------------------------
  // interrupt on INT1 (PD3) pin rising edge
  //MCUCR |= (1<<ISC11) | (1<<ISC10);
  
  // turn on external interrupt
  //GICR  |= (1<<INT1);

  /*PORTD |= (1 << PD5);
  PORTD |= (1 << PD6);
  OCR1A = 511;
  OCR1B = 511;*/
}


int main(void)
{    
	initPorts();  
	UARTInit();

	sei();

	UartPutS("start\r\n");			

    int i;

    while(1) {				
	  for (i=0; i < 30000; i++) _delay_us(10);
	  UartPutS("ping\r\n");		
	  OCR1A -= 30;  
      OCR1B -= 30;
	}

}

